package ims.data;

public class Stack implements Comparable<Stack> {

	private static final int INIT_STACK_SIZE=20; 
	
	private int nxt=0;
	private int[] _stack;
	
	private Stack(int size){
		_stack=new int[size];
	}
	
	public Stack(){
		_stack=new int[INIT_STACK_SIZE];
		_stack[0]=0;
		++nxt;
	}
	
	public Stack copy(){
		int len=this._stack.length;
		Stack s=new Stack(len);
		System.arraycopy(_stack, 0, s._stack, 0, nxt);
		s.nxt=nxt;
		return s;
	}
	
	public boolean equals(Stack o){
		if(nxt!=o.nxt) //Can't be same if they have different number of items
			return false;
		for(int i=0;i<nxt;++i)
			if(_stack[i]!=o._stack[i])
				return false;
		return true;
	}
	
	public int size(){
		return nxt;
	}
	
	public void add(int i){
		if(nxt==_stack.length){ //double the size of the stack
			int[] n=new int[_stack.length*2];
			System.arraycopy(_stack, 0, n, 0, _stack.length);
			_stack=n;
		}
		_stack[nxt++]=i;
	}
	
	public int pop(){
		return _stack[--nxt];
	}
	
	public int peek(int offset){
		int i=nxt-1-offset;
		if(i<0)
			return -1;
		else
			return _stack[i];
	}
	
	public String toString(){
		StringBuilder sb=new StringBuilder("[").append(_stack[0]);
		for(int i=1;i<nxt;++i)
			sb.append(", ").append(_stack[i]);
		return sb.append(']').toString();
	}

	@Override
	public int compareTo(Stack o) {
		int a=this.size();
		int b=o.size();
		if(a!=b) //With a=3 and b=5, we want a negative number (3<5)
			return b-a;
		//A stack is also smaller if the topmost item is smaller, or second topmost, etc.
		for(int i=0;i<this.size();++i){
			int p=this.peek(i);
			int q=o.peek(i);
			if(p!=q) //p=3 and q=5 makes p smaller, q-p=-2
				return q-p;
		}
		return 0;
	}
	
	public StringBuilder appendSignature(StringBuilder sb){
		for(int i=0;i<nxt;++i)
			sb.append((char) _stack[i]);
		return sb;
	}
}
