package ims.headfinder;

import ims.data.Instance;
import ims.hotcoref.symbols.SymbolTable;
import ims.perceptron.Long2IntInterface;
import ims.trans.InstanceMapping;
import ims.trans.labelfilter.ILabelFilter;
import ims.util.ArrayFunctions;
import ims.util.Pair;

public class Decoder {

	public final float[] parameters;
	public final Long2IntInterface l2i;
	public final SymbolTable symTab;
	public final InstanceMapping instanceMapping;
	public final FE fe;
	public final ILabelFilter labels;	
	
	public final String[] deprelLookup;
	
	private final long[] sink; //This guy makes all of this thread-*un*safe
	
	public final int lblBits;
	
//	private final int[] labels;
	
	
	public Decoder(float[] parameters, Long2IntInterface l2i, SymbolTable symTab, InstanceMapping instanceMapping,ILabelFilter labels) {
		this.parameters = parameters;
		this.l2i = l2i;
		this.symTab = symTab;
		this.instanceMapping = instanceMapping;
		this.deprelLookup = symTab.deprels.getInverted();
		this.sink=new long[FE.SINK_LEN];
		this.lblBits=symTab.deprels.getBits();
		this.fe=new FE(symTab);
		this.labels=labels;
//		int lblCount=0;
//		for(String s:deprelLookup)
//			if(s!=null && !s.equals(ISentence.ROOT))
//				++lblCount;
//		this.labels=new int[lblCount];
//		int q=0;
//		for(String s:deprelLookup)
//			if(s!=null && !s.equals(ISentence.ROOT))
//				this.labels[q++]=symTab.deprels.lookup(s);
	}
	
	public Decoder(Model m){
		this(m.parameters,m.l2i,m.symTab,m.instanceMapping,m.labels);
	}
	
	public String[] intLabels2String(int[] lbls){
		String[] r=new String[lbls.length];
		for(int dep=1;dep<r.length;++dep)
			r[dep]=deprelLookup[lbls[dep]];
		return r;
	}
	
	public Pair<int[],int[]> parse(Instance is){
		int[] heads=new int[is.form.length];
		int[] lbls=new int[is.form.length];
		heads[0]=-1;
		lbls[0]=2;
		for(int dep=1;dep<heads.length;++dep){
			HeadLabelScoreReturn r=findBestHead(is,dep);
			heads[dep]=r.head;
			lbls[dep]=r.label;
		}
		return new Pair<>(heads,lbls);
	}
	
	static class HeadLabelScoreReturn {
		public final double score;
		public final int head;
		public final int dep;
		public final int label;
		public HeadLabelScoreReturn(double score, int head, int dep, int label) {
			this.score = score;
			this.head = head;
			this.dep = dep;
			this.label = label;
		}
	}
	
	public HeadLabelScoreReturn findBestHead(Instance is,int dep){
		int bestHead=-1;
		int bestLabel=-1;
		double bestScore=-9999.d;
		int[] headCandidates=getHeadCandidatesOrdered(dep,is.head.length);
		for(int head:headCandidates){
			int features=fe.extractFeatures(is, head, dep, sink);
			ArrayFunctions.shiftAllLeftByBits(sink, lblBits, features);
			for(int label:labels.getAllowedLabels(head, dep, is)){
				double score=0.d;
				for(int i=0;i<features;++i){
					long l=sink[i];
					l|=label;
					int idx=l2i.l2i(l);
					score+=parameters[idx];
				}
				if(score>bestScore){
					bestHead=head;
					bestLabel=label;
					bestScore=score;
				}
			}
		}
		return new HeadLabelScoreReturn(bestScore,bestHead,dep,bestLabel);
	}

	//Since we have a good chance of ties with this smallish feature set, we want ties to be broken by selecting the closest head
	//This is accomplished by requiring strict increases in score in the findBestHead() method *and* iterating over possible heads according to the method below
	static int[] getHeadCandidatesOrdered(int dep,int length){
		int[] r=new int[length-1];
		int i=0;
		int d=1; //distance from dep
		for(;dep-d>=0 && dep+d<length;++d){
			r[i++]=dep-d;
			r[i++]=dep+d;
		}
		//At this point we have collected either all potential heads on the left OR on the right (sometimes both). Below we make up for the boundary cases.
		//In other words, only one of the loops below should fire
		while(dep-d>=0){
			r[i++]=dep-d;
			++d;
		}
		while(dep+d<length){
			r[i++]=dep+d;
			++d;
		}
		return r;
	}
}
