package ims.headfinder;

import java.io.IOException;

import ims.data.ISentence;
import ims.data.Instance;
import ims.io.ReaderWriterFactory;
import ims.io.SentenceReader;
import ims.io.SentenceWriter;
import ims.trans.Options;
import ims.trans.util.Eval;
import ims.util.Accuracy;
import ims.util.DBO;
import ims.util.Pair;
import ims.util.ZipUtil;

public class Test {

	public static void main(String[] args) throws ClassNotFoundException, IOException{
		Options options=new Options(args);
		Model m=ZipUtil.loadObjectFromEntry(Model.ZIP_ENTRY, options.model, Model.class);
		Decoder d=new Decoder(m);
		
		SentenceReader<?> reader=ReaderWriterFactory.getReader(options.test, options.format);
		SentenceWriter writer=ReaderWriterFactory.getWriter(options.out);
		
		test(reader,writer,d);
		
		if(options.gold!=null){
			Pair<Accuracy,Accuracy> a=Eval.getUASandLAS(options.out, options.gold, options.format);
			System.out.println("UAS: "+a.getLeft().toString() + "   LAS: "+a.getRight().toString());
		}
		options.done();
	}
	
	public static void test(SentenceReader<?> reader,SentenceWriter writer,Decoder decoder) throws IOException{
		int erase=0;
		int sentCount=0;
		long start=System.currentTimeMillis();
		DBO.printNoPrefix("Sent: ");
		for(ISentence s:reader){
			++sentCount;
			if(sentCount%111==1)
				erase=DBO.eraseAndPrint(erase, infoString(sentCount,start));
			Instance is=decoder.instanceMapping.createTestInstance(s, decoder.symTab);
			Pair<int[],int[]> p=decoder.parse(is);
			int[] heads=p.getLeft();
			String[] labels=decoder.intLabels2String(p.getRight());
			s.setPredHeads(heads);
			s.setPredLabels(labels);
			writer.write(s);
		}
		erase=DBO.eraseAndPrint(erase, infoString(sentCount,start));
		DBO.println();
		writer.close();
	}

	private static String infoString(int sentCount, long start) {
		long sps=(System.currentTimeMillis()-start)/sentCount;
		return String.format("%-6d    %d/s",sentCount,sps);
	}
}
