package ims.oracle;

import ims.oracle.DeterministicArcStandardSwapOldOracle.InOrderOrder;

public class BruteForceShiftOKCheck {

	private MinimalConfiguration config;
	private InOrderOrder inOrder;
	private int[] heads;
	
	private BruteForceShiftOKCheck(IConfiguration config,InOrderOrder inOrder,int[] heads){
		this.config=MinimalConfiguration.getMinimalConfigurationCopyShiftedByOne(config, heads.length);
		this.inOrder=inOrder;
		this.heads=heads;
	}
	
	public boolean canReachTerminal(){
		while(config.stack.size()>2){
			if(DeterministicArcStandardOracle.canLeftArc(heads, config)){
				config.doLA();
			} else if(DeterministicArcStandardOracle.canRightArc(heads, config)){
				config.doRA();
			} else if(inOrder!=null && DeterministicArcStandardSwapOldOracle.canSwap(heads, config, inOrder)){
				config.doSwap();
			} else if(config.bufferSize()>0){
				config.doShift();
			} else { //Hit dead end
				return false;
			}
		}
		return true;
	}
	
	private boolean canReachTerminalWithoutReswapping() { //Here we want to try without swapping s1 and s2 back on the buffer (note that it's already been shifted one step in place from the constructor -- this is to avoid the silly {SH,LA} ambiguitity that relies on everything being shifted back onto the buffer
		int initS1=config.peekStack(1);
		while(config.stack.size()>2){
			if(DeterministicArcStandardOracle.canLeftArc(heads, config)){
				config.doLA();
			} else if(DeterministicArcStandardOracle.canRightArc(heads, config)){
				config.doRA();
			} else if(config.peekStack(1)!=initS1 && DeterministicArcStandardSwapOldOracle.canSwap(heads, config, inOrder)){
				config.doSwap();
			} else if(config.bufferSize()>0){
				config.doShift();
			} else { //Hit dead end
				return false;
			}
		}
		return true;
	}
	
	public static boolean canShift(IConfiguration c,InOrderOrder inOrder,int[] heads){
		if(c.bufferSize()==0)
			return false;
		BruteForceShiftOKCheck check=new BruteForceShiftOKCheck(c, inOrder, heads);
		return check.canReachTerminal();
	}
	
	public static boolean canShiftWithoutReswapping(IConfiguration c,InOrderOrder inOrder,int[] heads){
		if(c.bufferSize()==0)
			return false;
		BruteForceShiftOKCheck check=new BruteForceShiftOKCheck(c, inOrder, heads);
		return check.canReachTerminalWithoutReswapping();
	}

}
