package ims.oracle;

import ims.data.Instance;
import ims.oracle.Configuration.TR;
import ims.trans.Options;
import ims.trans.Options.TransitionSystem;

import java.util.Arrays;
import java.util.List;

public class DelayDynamicOracle extends AbstractDynamicOracle{

	private final int dynIter;
	
	private final AbstractStaticOracle detOracle;
	private final AbstractDynamicOracle dynOracle;
	
	protected DelayDynamicOracle(TR[] myTRs) {
		super(myTRs);
		this.dynIter=Options.dynDelayIter;
		this.detOracle=OracleFactory.getStaticOracle(Options.statSystem);
		this.dynOracle=OracleFactory.getDynamicOracle(Options.dynSystem);
		if(this.detOracle.canDoNonProj()!=this.dynOracle.canDoNonProj() || !Arrays.equals(detOracle.myTRs, dynOracle.myTRs)){
			System.err.println("Incompaticale static and dynamic oracles?");
			System.err.println("Static:  "+Options.statSystem);
			System.err.println("Dynamic: "+Options.dynSystem);
			System.exit(1);
		}
	}

	@Override
	public List<String> getCanonicalSequence(int[] heads, String[] labels, boolean labeled) {
		detOracle.initSentence(heads);
		return detOracle.getCanonicalSequence(heads, labels, labeled);
	}

	@Override
	public boolean canDoNonProj() {
		return detOracle.canDoNonProj();
	}

	@Override
	public boolean isCorrectULBLTransition(TR tr, IConfiguration conf,Instance inst) {
		IOracle oracle=getCurrentOracle();
		boolean correctULBL=oracle.isCorrectULBLTransition(tr, conf, inst);
		return correctULBL;
	}

	@Override
	public void initSentence(int[] heads) {
		detOracle.initSentence(heads);
		dynOracle.initSentence(heads);
	}
	
	private IOracle getCurrentOracle(){
		if(curIter<dynIter)
			return detOracle;
		else
			return dynOracle;
	}

	@Override
	public TransitionSystem getTS() {
		return getCurrentOracle().getTS();
	}
}
