package ims.oracle;

import java.util.ArrayList;
import java.util.List;

import ims.data.Instance;
import ims.oracle.Configuration.TR;
import ims.trans.Options.TransitionSystem;

public class DeterministicArcStandardLazyOracle extends AbstractStaticOracle {

	protected DeterministicArcStandardLazyOracle(TR[] myTRs) {
		super(myTRs);
	}

	@Override
	public List<String> getCanonicalSequence(int[] heads, String[] labels,	boolean labeled) {
		Configuration c=new Configuration(heads);
		List<String> operations=new ArrayList<String>();
		while(!c.isTerminal()){
			final String op;
			if(DeterministicArcStandardOracle.canRightArc(heads,c)){
				op=(labeled?TR.RAS+"_"+labels[c.peekStack(0)]:TR.RAS.toString());
				c.doRightArcS();
			} else if(DynamicArcStandardOracle.canShift(heads, c)){
				op=TR.SH.toString();
				c.doShift();
			} else if(DeterministicArcStandardOracle.canLeftArc(heads, c)){
				op=(labeled?TR.LA+"_"+labels[c.peekStack(1)]:TR.LA.toString());
				c.doLeftArc();
			} else {
				throw new Error("hit dead end state, something is fundamentally wrong here");
			}
			operations.add(op);
		}
		return operations;
	}

	@Override
	public boolean canDoNonProj() {
		return false;
	}

	@Override
	public boolean isCorrectULBLTransition(TR tr, IConfiguration conf,	Instance inst) {
		int[] heads=inst.head;
		switch(tr){
		case LA:  return !DynamicArcStandardOracle.canShift(heads, conf) && DeterministicArcStandardOracle.canLeftArc(heads, conf); 
		case RAS: return DeterministicArcStandardOracle.canRightArc(heads, conf);
		case SH:  return DynamicArcStandardOracle.canShift(heads, conf);
		default: throw new Error("not supported by this system");
		}
	}

	@Override
	public void initSentence(int[] heads) {
	}

	@Override
	public TransitionSystem getTS() {
		return TransitionSystem.ArcSLazy;
	}

}
