package ims.oracle;

import ims.data.Instance;
import ims.oracle.Configuration.TR;
import ims.oracle.DeterministicArcStandardSwapOldOracle.InOrderOrder;
import ims.trans.Options;
import ims.trans.Options.TransitionSystem;

import java.util.List;

public class DynamicArcStandardSwapLASWDynOracle extends AbstractDynamicOracle {

	protected DynamicArcStandardSwapLASWDynOracle(TR[] myTRs) {
		super(myTRs);
	}

	private InOrderOrder inOrder;

	@Override
	public void initSentence(int[] heads){ //Need this for swap stuff
		inOrder=new InOrderOrder(InOrderOrder.getSequence(heads));
	}

	private final IOracle detOracle=OracleFactory.getStaticOracle(Options.TransitionSystem.ArcSNewSwap);
	@Override
	public List<String> getCanonicalSequence(int[] heads, String[] labels,boolean labeled) {
		return detOracle.getCanonicalSequence(heads, labels, labeled);
	}

	@Override
	public boolean canDoNonProj() {
		return true;
	}

	@Override
	public boolean isCorrectULBLTransition(TR tr, IConfiguration conf,	Instance inst) {
		int[] heads=inst.head;
		if(tr==TR.SH){
			if(DeterministicArcStandardOracle.canLeftArc(heads, conf) || DeterministicArcStandardSwapOldOracle.canSwap(heads, conf, inOrder))
				return BruteForceShiftOKCheck.canShift(conf, inOrder, heads);
			if(DeterministicArcStandardOracle.canRightArc(heads, conf))
				return false;
			else
				return conf.bufferSize()>0;
		}
		switch(tr){
		case LA:  return DeterministicArcStandardOracle.canLeftArc(heads, conf);
		case RAS: return DeterministicArcStandardOracle.canRightArc(heads, conf);
		case SW:  return DeterministicArcStandardSwapOldOracle.canSwap(heads, conf, inOrder);
		default:  throw new Error("not supported");
		}
	}
	
	@Override
	public TransitionSystem getTS() {
		return TransitionSystem.ArcSLASWDynSwap;
	}

}
