package ims.oracle;

import ims.data.Instance;
import ims.oracle.Configuration.TR;
import ims.oracle.DeterministicArcStandardSwapOldOracle.InOrderOrder;
import ims.trans.Options;
import ims.trans.Options.TransitionSystem;
import ims.util.DisjointSetZeroToN;

import java.util.List;

public class DynamicSentenceSplittingOracle extends AbstractDynamicOracle {

	private InOrderOrder inOrder;
	private DisjointSetZeroToN mpcs;
	
	protected DynamicSentenceSplittingOracle(TR[] myTRs) {
		super(myTRs);
	}

	
	private final IOracle deterministicOracle=OracleFactory.getStaticOracle(Options.TransitionSystem.ArcSNewSwap);
	
	@Override
	public List<String> getCanonicalSequence(int[] heads, String[] labels, boolean labeled) {
		return deterministicOracle.getCanonicalSequence(heads, labels, labeled);
	}

	@Override
	public boolean canDoNonProj() {
		return true;
	}

	@Override
	public boolean isCorrectULBLTransition(TR tr, IConfiguration conf, Instance inst) {
		int[] heads=inst.head;
		switch(tr){
		case LA:  return DeterministicArcStandardOracle.canLeftArc(heads, conf);
		case RAS: return DeterministicArcStandardOracle.canRightArc(heads, conf);
		case SW:  return DeterministicArcStandardSwapNewOracle.canSwap(heads, conf, inOrder, mpcs);
		default:  //Do nothing, go through
		}

//		int lastSB=conf.getLastSB();
		boolean canSB=conf.sbPermissible() && inst.sentenceBegin.contains(conf.peekBuffer(0));
//		boolean canSB=conf.getBufferNoPushedBack()==0 && lastSB<conf.peekBuffer(0) && inst.sentenceBegin.contains(conf.peekBuffer(0));
		if(tr==TR.SB)
			return canSB;
		//Else it's shift, which is allowed *exactly* when none of the others are allowed
		return !canSB && !DeterministicArcStandardOracle.canLeftArc(heads, conf) && !DeterministicArcStandardOracle.canRightArc(heads, conf) && !DeterministicArcStandardSwapNewOracle.canSwap(heads, conf, inOrder, mpcs);
		
		
//		
//		//This is the logic for canSB: not blocked, next token starts a next sentence, and we didn't just do an SB, and the previous tr is not null (initial state)
////		boolean canSB=!conf.isBlockedForShift() && inst.sentenceBegin.contains(conf.peekBuffer(0)) && conf.getPrevTR()!=TR.SB && conf.getPrevTR()!=null;
//		//This is the logic for canSH: not blocked, and there is nothing else we can do
////		boolean canSH=!conf.isBlockedForShift() && !DeterministicArcStandardOracle.canLeftArc(heads, conf) && !DeterministicArcStandardOracle.canRightArc(heads, conf) && !DeterministicArcStandardSwapNewOracle.canSwap(heads, conf, inOrder, mpcs) && !canSB;
//		
//		//We can do some of these jointly:
//		if(conf.isBlockedForShift())
//			return false;
//		
//		boolean canSB=inst.sentenceBegin.contains(conf.peekBuffer(0)) && conf.getPrevTR()!=TR.SB && conf.getPrevTR()!=null && conf.getBufferNoPushedBack()==0;
//		
//		if(tr==TR.SB)
//			return canSB;
//		if(tr==TR.SH)
//			return !canSB && !DeterministicArcStandardOracle.canLeftArc(heads, conf) && !DeterministicArcStandardOracle.canRightArc(heads, conf) && !DeterministicArcStandardSwapNewOracle.canSwap(heads, conf, inOrder, mpcs);
//		
//		return false;
	}

	@Override
	public void initSentence(int[] heads) {
		inOrder=new InOrderOrder(InOrderOrder.getSequence(heads));
		mpcs=DeterministicArcStandardSwapNewOracle.getMPCs(heads,new Configuration(heads));
	}

	@Override
	public TransitionSystem getTS() {
		return TransitionSystem.ArcSNewSwap;
	}

}
