package ims.oracle;

import ims.data.ISentence;
import ims.data.Instance;
import ims.io.NonProjDiscardingReader;
import ims.io.ReaderWriterFactory;
import ims.io.SentenceReader;
import ims.oracle.Configuration.TR;
import ims.trans.Options;
import ims.trans.Train;
import ims.util.IOUtils;
import ims.util.MutableInt;
import ims.util.Util;

import java.io.BufferedWriter;
import java.io.File;
import java.io.IOException;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collections;
import java.util.HashSet;
import java.util.List;
import java.util.Map;
import java.util.Map.Entry;
import java.util.Set;
import java.util.TreeMap;
import java.util.TreeSet;

public class PlotOracleLattice {

	public static boolean VERBOSE_ERRORS=true;
	
	
	
	
	//
	//XXX
	//
	// If we hit dead ends, then it would make sense to continue to allow shift and see we're we end up.
	// Though they should be colored differently. What is really on a dead end path should be red, and the remainder different...
	//
	// To do this, do the color flagging stuff (flagDeadEnds). Then start from the list of final states that are not terminal and continue. Continue meaning, force a (or multiple?) shifts and see if it finds the way to a proper terminal state, or if it hits end of buffer with too much on the stack.
	
	//Bookkeeping how things are going.
	static int[][] MAX_PARALLEL_STATES_HISTO=new int[1000][1000];
	static Map<Integer,MutableInt> PATHS_HISTO=new TreeMap<Integer,MutableInt>();
	static final int MAX_PARALLEL_TH=20;
	static int globalMaxParallel=0;
	static int countAboveTH=0;

	
	public static final boolean BREAK_ON_FAIL=true;
	
	static int sentId=0;
	static long totalTransitions=0;
	static long totalStates=0;
	static int sumX=0;
	
	static Map<String,MutableInt> seenOptions=new TreeMap<String,MutableInt>();
	static Map<String,MutableInt> deadAmbiguityCreators=new TreeMap<String,MutableInt>();
	public static long finalNonTerminal=0;
	public static int sentWOFinalTerminal=0;
	public static int finalTerminal=0;
	public static int sentAllGood=0;
	public static int sentDeadEndButTerminal=0;
	public static int deadEndExtendedStateCount=0;
	
	
	public static void main(String[] args) throws IOException{
		long start=System.currentTimeMillis();
		Options options=new Options(args);

		String dotPrefix=options.dotPrefix;
		SentenceReader<? extends ISentence> reader=ReaderWriterFactory.getReader(options.train, options.format.sentenceClass);
		if(options.discardNonProj)
			reader=NonProjDiscardingReader.createReader(reader);

		IOracle oracle=OracleFactory.getVanillaOracle(options.oracle, options.transitionSystem);
		oracle=Train.sanityCheckProjectivity(options, reader, oracle);
		
//		int firstNPSent=Train.findNPSent(reader);
//		
//		if(!oracle.canDoNonProj()){
//			Train.failCantDoNonProj(firstNPSent, oracle);
//		}
		
		if(dotPrefix!=null && !IOUtils.createParentFolderIfNecessary(dotPrefix)){
			System.err.println("Failed to find or create dot folder: "+dotPrefix);
			System.err.println("ABORTING");
			System.exit(1);
		}
		
		for(ISentence s:reader){
			int[] head=s.getGoldHeads();
			oracle.initSentence(head);
//			List<List<Edge>> edges=oracle.derive(s.head);
			LatticeReturn lr=derive(head,oracle);
			sumX+=lr.finalCorrect.x;
//			reverseEdgesAndFlagDeadEnds(edges);
			flagDeadEnds(lr);
			if(dotPrefix!=null)
//				printDotFile(dotPrefix,edges,sentId,oracle,s.head);
				printDotFile(dotPrefix,lr,sentId,oracle,head);
			sentId++;
			if(BREAK_ON_FAIL && (finalNonTerminal>0 || sentWOFinalTerminal>0))
				break;
			if(sentId>=options.count)
				break;
		}
		System.out.println("Global max parallel states: "+globalMaxParallel);
		System.out.println("Above TH "+MAX_PARALLEL_TH+": "+countAboveTH);
		System.out.println("Total transitions: "+Util.insertCommas(totalTransitions,false));
		System.out.println("Total states: "+Util.insertCommas(totalStates,false));
		System.out.println("Total states leading to dead end: "+Util.insertCommas(deadEndExtendedStateCount, false));
		System.out.println("Total OK states: "+Util.insertCommas(totalStates-deadEndExtendedStateCount, false));
		System.out.println("Final states non-terminal: "+Util.insertCommas(finalNonTerminal, false));
		System.out.println("Sentences with terminal final states:        "+Util.insertCommas(finalTerminal, false));
		System.out.println("Sentences with terminal, but also dead ends: "+Util.insertCommas(sentDeadEndButTerminal,false));
		System.out.println("Sentences without terminal final states:     "+Util.insertCommas(sentWOFinalTerminal, false));
		System.out.println("Sentences all good:                          "+Util.insertCommas(sentAllGood,false));
		System.out.println("Sum shortest path transitions:               "+Util.insertCommas(sumX, false));
		System.out.println("Total sentences:                             "+sentId);
		System.out.println();
		System.out.println("Seen options:");
		for(Entry<String,MutableInt> e:seenOptions.entrySet()){
			System.out.printf("%-20s  %d\n",e.getKey(),e.getValue().getValue());
		}
		System.out.println();
		System.out.println("Dead end ambiguities:");
		for(Entry<String,MutableInt> e:deadAmbiguityCreators.entrySet()){
			System.out.printf("%-20s  %d\n",e.getKey(),e.getValue().getValue());
		}		
		long time=System.currentTimeMillis()-start;
		System.out.println();
		System.out.println("Total time: "+Util.insertCommas(time, true));
	}

	private static void flagDeadEnds(LatticeReturn lr){
		if(lr.finalCorrect==null){
			flagAllDeadEnd(lr);
			return;
		}
		//First flag all OK states
		{
			List<Configuration> agenda=new ArrayList<Configuration>();
			agenda.add(lr.finalCorrect);
			while(!agenda.isEmpty()){
				List<Configuration> newAgenda=new ArrayList<Configuration>();
				for(Configuration to:agenda){
					for(Entry<Configuration,TR> e:to.incoming.entrySet()){
						Configuration from=e.getKey();
						if(from.deadEndState!=1){ //If it's 1 then we've already been here and it's OK, so we don't need to go down this path again
							newAgenda.add(from);
							from.deadEndState=1;
						}
					}
				}
				agenda=newAgenda;
			}
		}
		//Now start from all dead ends and go only until the first one flagged 1
		for(Configuration finnal:lr.finalConfigs){
			if(finnal==lr.finalCorrect) //Skip the correct
				continue;
			List<Configuration> agenda=new ArrayList<Configuration>();
			agenda.add(finnal);
			while(!agenda.isEmpty()){
				List<Configuration> newAgenda=new ArrayList<Configuration>();
				for(Configuration to:agenda){
					for(Entry<Configuration,TR> e:to.incoming.entrySet()){
						Configuration from=e.getKey();
						if(from.deadEndState==0){ //If it's 1 then we've already been here and it's OK. If it's 2 we've also already been here and it's not ok. In either case we don't need to go down this path again.
							newAgenda.add(from);
							from.deadEndState=2;
						}
					}
				}
				agenda=newAgenda;
			}
		}
	}
	
	/*
	 * Flags all as dead ends -- this happens when there are no terminal correct states
	 */
	private static void flagAllDeadEnd(LatticeReturn lr) {
		List<Configuration> agenda=new ArrayList<Configuration>();
		agenda.add(lr.init);
		while(!agenda.isEmpty()){
			List<Configuration> newAgenda=new ArrayList<Configuration>();
			for(Configuration c:agenda){
				for(Entry<TR,Configuration> e:c.outgoing.entrySet()){
					Configuration to=e.getValue();
					if(to.deadEndState!=2){
						to.deadEndState=2;
						newAgenda.add(to);
					}
				}
			}
			agenda=newAgenda;
		}
		lr.init.deadEndState=2; //Don't forget about init
	}
//
//	private static void flagDeadEndsOLD(LatticeReturn lr) { //This one does it in the old less fast way (basically you can visit the same state more than once if you didn't start with the final correct state)
//		for(Configuration last:lr.finalConfigs){
//			List<Configuration> agenda=new ArrayList<Configuration>();
//			agenda.add(last);
//			while(!agenda.isEmpty()){
//				List<Configuration> newAgenda=new ArrayList<Configuration>();
//				for(Configuration to:agenda){
//					for(Entry<Configuration,TR> e:to.incoming.entrySet()){
//						Configuration from=e.getKey();
//						if(from.deadEndState!=1) //If it's 1 then we've already been here and it's OK, so we don't need to go down this path again
//							newAgenda.add(from);
//						if(to.deadEndState==2 && from.deadEndState==0){ //Means to is dead end and from has not been visited at all yet
//							from.deadEndState=2;
//						}
//						if(to.deadEndState<2)
//							from.deadEndState=1;
//					}
//				}
//				agenda=newAgenda;
//			}
//		}
//	}

	private static void printDotFile(String dotPrefix, LatticeReturn lr,int i,IOracle oracle, int[] head) throws IOException {
		File output=new File(dotPrefix+"."+i+".dot");
		BufferedWriter bw=IOUtils.getBufferedWriter(output);
		bw.write("digraph G {\n");
		bw.write("rankdir=LR;\n");
		Set<String> doneNodes=new HashSet<String>();
		List<Configuration> agenda=Collections.singletonList(lr.init);
		while(!agenda.isEmpty()){
			List<Configuration> newAgenda=new ArrayList<Configuration>();
			for(Configuration from:agenda){
				String c1=getNodeIdentifier(from);
				if(!doneNodes.contains(c1)){
					doneNodes.add(c1);
					bw.write(getNodeDef(from,oracle,head));
				}
				boolean anyBadFork=false;
				for(Entry<TR,Configuration> e:from.outgoing.entrySet()){
					Configuration to=e.getValue();
					if(from.deadEndState==1 && to.deadEndState==2)
						anyBadFork=true;
					if(!doneNodes.contains(getNodeIdentifier(to)))
						newAgenda.add(to);
					String c2=getNodeIdentifier(to);
					if(!doneNodes.contains(c2)){
						doneNodes.add(c2);
						bw.write(getNodeDef(to,oracle,head));
					}
//					bw.write(c1+" -> "+c2+" [label="+e.getKey().toString()+"; color=\""+e.getKey().htmlColor+"\"; penwidth=3];\n");					
					bw.write(c1+" -> "+c2+" [label="+e.getKey().toString()+" color=\""+e.getKey().htmlColor+"\" penwidth=3];\n");
				}
				if(anyBadFork)
					recordBadForks(from);
			}
			agenda=newAgenda;
		}
		
		bw.write("}\n");
		bw.close();
	}

	
	private static void recordBadForks(Configuration from) {
		Set<TR> options=new TreeSet<TR>();
		Set<TR> good=new TreeSet<TR>();
		Set<TR> bad=new TreeSet<TR>();
		for(Entry<TR,Configuration> e2:from.outgoing.entrySet()){
			TR tr=e2.getKey();
			boolean isGood=e2.getValue().deadEndState==1;
			options.add(tr);
			if(isGood)
				good.add(tr);
			else
				bad.add(tr);
		}
		StringBuilder sb=new StringBuilder("{").append(Util.joinStrings(options, ",")).append("}");
		sb.append("--BAD:").append(Util.joinStrings(bad, ","));
		MutableInt.incrementMapValue(deadAmbiguityCreators, sb.toString());
		
	}

	private static String getNodeIdentifier(Configuration conf) {
		return "c"+conf.x+"_"+conf.y;
	}

	private static String getNodeDef(Configuration conf,IOracle oracle, int[] head) {
		String id=getNodeIdentifier(conf);
		StringBuilder sb=new StringBuilder(id).append(" [label=\"").append(id).append("\\n");
		sb.append(conf.bufferToString()).append("\\n");
		sb.append(conf.stackToString()).append("\\n");
		sb.append(Arrays.toString(conf.getPredHeads())).append("\\n");
		String os="{}";
		sb.append(os);
		sb.append("\" ");
		if(conf.isTerminal()){
			sb.append("shape=doubleoctagon ");
		} else {
			os=getPermissibleActions(conf,head,oracle);
		}
		
		MutableInt.incrementMapValue(seenOptions, os);

		if(conf.deadEndState==2){
			sb.append("color=\"#ff0000\" ");
			deadEndExtendedStateCount++;
		}
		sb.append("];\n");
		return sb.toString();
	}

//	private static List<String> extractPaths(List<List<Edge>> edges){
//		List<LinkedList<Edge>> agenda=new ArrayList<LinkedList<Edge>>();
//		{
//			LinkedList<Edge> init=new LinkedList<Edge>();
//			List<Edge> last=edges.get(edges.size()-1);
//			if(last.size()>1)
//				throw new Error("!"); //only one way to reach terminal state
//			init.addFirst(last.get(0));
//			agenda.add(init);
//		}
////		suffixes.add();
//		List<LinkedList<Edge>> next=new ArrayList<LinkedList<Edge>>();
//		for(int i=edges.size()-2;i>=0;--i){
//			for(Edge e:edges.get(i)){
//				for(LinkedList<Edge> suffix:agenda){
//					if(e.toState==suffix.peekFirst().fromState){
//						LinkedList<Edge> n=new LinkedList<Edge>(suffix);
//						n.addFirst(e);
//						next.add(n);
//					}
//				}
//			}
//			agenda=next;
//			next=new ArrayList<LinkedList<Edge>>();
//		}
//		List<String> r=new ArrayList<String>();
//		for(LinkedList<Edge> l:agenda){
//			StringBuilder sb=new StringBuilder();
//			for(Edge e:l)
//				sb.append(e.t.toString()).append(" ");
//			r.add(sb.toString().trim());
//		}
//		return r;
//	}

	
	public static class LatticeReturn {
		public Configuration init;
		public Set<Configuration> finalConfigs=new TreeSet<Configuration>();
		public Configuration finalCorrect;
		public LatticeReturn(Configuration init){
			this.init=init;
		}
	}

	public static LatticeReturn derive(int[] heads,IOracle oracle){
		Configuration start=new Configuration(heads);
		LatticeReturn lr=new LatticeReturn(start);
		start.x=0;
		start.y=1;
		List<Configuration> agenda=new ArrayList<Configuration>();
		agenda.add(start);
		Map<String,Configuration> sig2state=new TreeMap<String,Configuration>();
		Set<Configuration> newItems=new TreeSet<Configuration>();
		List<List<Configuration>> allStates=new ArrayList<List<Configuration>>();
		int x=1;
		int y=1;
		boolean anyFinalTerminal=false;
		boolean hasDeadEnd=false;
		while(!agenda.isEmpty()){
			List<Configuration> processed=new ArrayList<Configuration>();
			for(Configuration c:agenda){
				PlotOracleLattice.totalStates++;
				processed.add(c);
				List<TR> ops=oracle.getActions(c,makePhonyInst(heads));
				if(ops.isEmpty()){
					if(!c.isTerminal()){
						if(VERBOSE_ERRORS)
							System.err.println("Terminating at non-terminal state in sentence "+sentId);
						PlotOracleLattice.finalNonTerminal++;
						hasDeadEnd=true;
						c.deadEndState=2;
						lr.finalConfigs.add(c);
					} else {
						throw new Error("this shouldn't happen -- should be filtered out below where we reseed the agenda.");
					}
				}
				for(TR tr:ops){
					Configuration n=c.copy();
					n.applyTR(tr);
					String stateSig=n.getStateSignature();
					Configuration o=sig2state.get(stateSig);
					if(o!=null && o.hashCode()!=n.hashCode()){
						System.out.println("unequal hashcodes!");
//						String otherSig=o.getStateSignature();
//						if(otherSig.equals(stateSig))
//							System.out.println("!");
//						if(o.stack.equals(n.stack))
//							System.out.println("equal stacks");
//						if(o.buffer.equals(n.buffer))
//							System.out.println("equal buffers");
//						if(o.treeSig.equals(n.treeSig))
//							System.out.println("equal treesig");
//						StringBuilder oSig=new StringBuilder();
//						StringBuilder nSig=new StringBuilder();
//						StringBuilder sbOStack=new StringBuilder();
//						StringBuilder sbNStack=new StringBuilder();
//						StringBuilder sbOBuffer=new StringBuilder();
//						StringBuilder sbNBuffer=new StringBuilder();
//						StringBuilder sbOHeadSig=new StringBuilder();
//						StringBuilder sbNHeadSig=new StringBuilder();
//
//						o.treeSig.appendHeadSignature(oSig).append('|');
//						o.stack.appendSignature(oSig).append('|');
//						o.buffer.appendSignature(oSig).append('|');
//
//						n.treeSig.appendHeadSignature(nSig).append('|');
//						n.stack.appendSignature(nSig).append('|');
//						n.buffer.appendSignature(nSig).append('|');
//
//						o.stack.appendSignature(sbOStack);
//						n.stack.appendSignature(sbNStack);
//						o.buffer.appendSignature(sbOBuffer);
//						n.buffer.appendSignature(sbNBuffer);
//						o.treeSig.appendHeadSignature(sbOHeadSig);
//						n.treeSig.appendHeadSignature(sbNHeadSig);
//						
//						if(oSig.equals(nSig))
//							System.out.println("equal sigs (sb)");
//						if(oSig.toString().equals(nSig.toString()))
//							System.out.println("equal sigs (string)");
//						if(sbOStack.toString().equals(sbNStack.toString()))
//							System.out.println("equal stacks (string)");						
//						if(sbOBuffer.toString().equals(sbNBuffer.toString()))
//							System.out.println("equal buffers (string)");
//
//						if(sbOStack.append(sbOBuffer).toString().equals(sbNStack.append(sbNBuffer).toString()))
//							System.out.println("equal stack+buffer");
//						
////						System.out.println(oSig.length());
////						System.out.println(nSig.length());
////						System.out.println(oSig.toString().length());
////						System.out.println(nSig.toString().length());
//
//
//						
////						for(int i=0;i<oSig.length();++i)
////							System.out.print(oSig.charAt(i));
////						System.out.println();
////
////						for(int i=0;i<oSig.length();++i)
////							System.out.print(nSig.charAt(i));
////						System.out.println();
//						
//						
					}
					if(o!=null && !o.equals(n)){
						System.out.println("o and n different! S");
						throw new Error("!");
					}
					if(o==null){
						sig2state.put(stateSig, n);
						newItems.add(n);
						n.x=x;
						n.y=y++;
					} else {
//						o.mergeHasSwapped(n);
						n=o;
					}
					PlotOracleLattice.totalTransitions++;
					n.incoming.put(c, tr);
					c.outgoing.put(tr, n);
				}
			}
			
			allStates.add(processed);
			
			//Seed agenda
			agenda.clear();
			for(Configuration c:newItems){
				if(!c.isTerminal()){
					agenda.add(c);
				} else {
					if(lr.finalCorrect!=null){
						System.out.println("Contains? "+lr.finalConfigs.contains(c));
						throw new Error("This shouldn't happen...");
					}
					lr.finalConfigs.add(c);
					anyFinalTerminal=true;
					lr.finalCorrect=c;
				}
			}
			newItems.clear();
			
			//Reset x and y
			x++;
			y=1;
		}
		if(!anyFinalTerminal){
			PlotOracleLattice.sentWOFinalTerminal++;
			if(VERBOSE_ERRORS)
				System.err.println("Reached no final terminal state for sent "+sentId);
		} else {
			PlotOracleLattice.finalTerminal++;
			PlotOracleLattice.totalStates++;
		}
		
		if(!hasDeadEnd && anyFinalTerminal)
			PlotOracleLattice.sentAllGood++;
		
		if(hasDeadEnd && anyFinalTerminal)
			PlotOracleLattice.sentDeadEndButTerminal++;
		
		//Count the parallelism in the states
		{
			int totalStates=allStates.size();
			int singleStates=0;
			int maxParallel=0;
			for(int i=0;i<allStates.size();++i){
				List<Configuration> l=allStates.get(i);
				if(l.size()==1)
					singleStates++;
				maxParallel=Math.max(maxParallel,l.size());
			}
			if(maxParallel>=MAX_PARALLEL_STATES_HISTO[0].length)
				maxParallel=MAX_PARALLEL_STATES_HISTO[0].length-1;
			MAX_PARALLEL_STATES_HISTO[heads.length][maxParallel]++;
			globalMaxParallel=Math.max(globalMaxParallel, maxParallel);
			if(maxParallel>=MAX_PARALLEL_TH){
//				System.out.printf("%-10d  %d\n",sentId,maxParallel);
				countAboveTH++;
			}
		}
//		return allEdges;
		return lr;
	}

	private static Instance makePhonyInst(int[] heads) {
		return new Instance(heads);
	}

	public static String getPermissibleActions(IConfiguration conf,int[] heads,IOracle oracle){
		List<TR> ops=oracle.getActions(conf,makePhonyInst(heads));
		if(ops.size()==0)
			return "{}";
		StringBuilder sb=new StringBuilder("{").append(ops.get(0).toString());
		for(int i=1;i<ops.size();++i)
			sb.append(",").append(ops.get(i).toString());
		return sb.append("}").toString();
	}
}
