package ims.tag;

import gnu.trove.procedure.TIntProcedure;
import gnu.trove.set.TIntSet;
import gnu.trove.set.hash.TIntHashSet;

import java.util.Arrays;

public class SamplingTagContext implements ITagContext {

	final int[] tags;
	final int[] modifications;
	final TIntSet modified;
	
	public SamplingTagContext(int[] init) {
		tags=Arrays.copyOf(init, init.length);
		modifications=new int[init.length];
		modified=new TIntHashSet();
	}

	@Override
	public int getTag(int i) {
		if(i<1 || i>=tags.length)
			return NONE;
		else if(modifications[i]>0)
			return modifications[i];
		else
			return tags[i];
	}
	
	public void modify(int idx,int newTag){
		modifications[idx]=newTag;
		modified.add(idx);
	}
	
	public void rollbackModifications(){
		modified.forEach(rollbackProcedure);
		modified.clear();
	}
	
	public void commitModifications(){
		modified.forEach(commitProcedure);
		modified.clear();
	}

	TIntProcedure rollbackProcedure=new TIntProcedure() {
		@Override
		public boolean execute(int arg0) {
			modifications[arg0]=0;
			return true;
		}
	};
	
	TIntProcedure commitProcedure=new TIntProcedure() {
		@Override
		public boolean execute(int arg0) {
			tags[arg0]=modifications[arg0];
			modifications[arg0]=0;
			return true;
		}
	};

	public int length() {
		return tags.length;
	}

	public int[] getTags() {
		return tags;
	}
}
