package ims.tag;

import ims.data.ISentence;
import ims.hotcoref.symbols.AbstractSymbolMapping;
import ims.hotcoref.symbols.TagSymbolTable;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;
import java.util.regex.Pattern;

public class TagInstanceCreator implements Serializable {

	private static final long serialVersionUID = 1720401399902629486L;

	public static final int NONE=AbstractSymbolMapping.NONE_IDX;

	static final Pattern DIGIT_PATTERN=Pattern.compile("\\d");
	
	public final int affixLength;
	
	public TagInstanceCreator(int affixLength){
		this.affixLength=affixLength;
	}
	
	public <T extends ISentence> TagInstance[] registerTypesAndCreateTrainInstances(Iterator<T> it,TagSymbolTable symTab){
		List<TagInstance> instances=new ArrayList<>();
		while(it.hasNext()){
			T sent=it.next();
			TagInstance inst=createInstance(sent,symTab,true);
			instances.add(inst);
		}
		TagInstance[] r=instances.toArray(new TagInstance[instances.size()]);
		symTab.freezeMappings();
		return r;
	}
	
	public <T extends ISentence> TagInstance createTestInstance(T sent,TagSymbolTable symTab){
		return createInstance(sent,symTab,false);
	}
	
	private <T extends ISentence> TagInstance createInstance(T sent,TagSymbolTable symTab,boolean train){
		TagInstance inst=new TagInstance(sent.getLength());
		String[] forms=sent.getForms();
		for(int i=1;i<forms.length;++i){
			String form=forms[i];
			inst.forms[i]=symTab.forms.lookup(form);
			inst.prefixes[i]=new int[affixLength+1];
			inst.suffixes[i]=new int[affixLength+1];
			for(int s=affixLength;s>0;--s){
				if(form.length()>s){
					String suffix=form.substring(form.length()-s);
					inst.suffixes[i][s]=symTab.affixes.lookup(suffix);
					String prefix=form.substring(0, s);
					inst.prefixes[i][s]=symTab.affixes.lookup(prefix);
				} else {
					inst.suffixes[i][s]=NONE;
					inst.prefixes[i][s]=NONE;
				}
				inst.containsHyphen[i]=form.contains("-");
				inst.containsDigit[i]=DIGIT_PATTERN.matcher(form).matches();
				inst.containsUppercase[i]=containsUppercase(form);
			}
		}
		if(train){
			String[] tags=sent.getTaggerTrainPOS();
			for(int i=1;i<tags.length;++i)
				inst.tags[i]=symTab.tags.addSymbol(tags[i]);
		}
		return inst;
	}

	private static boolean containsUppercase(String form) {
		for(int i=0;i<form.length();++i)
			if(Character.isUpperCase(form.charAt(i)))
				return true;
		return false;
	}
	
}
