package ims.tag;

import ims.data.ISentence;
import ims.hotcoref.symbols.TagSymbolTable;
import ims.io.ReaderWriterFactory;
import ims.io.SentenceReader;
import ims.io.SentenceWriter;
import ims.util.Accuracy;

import java.io.File;
import java.io.IOException;
import java.util.Arrays;
import java.util.Iterator;

public class Test {

	public static void main(String[] args) throws ClassNotFoundException, IOException{
		TagOptions options=new TagOptions(args);
		
		testModel(options.model,options.test,options.out);
		
		if(options.gold!=null)
			eval(options.gold,options.out);
	}

	public static void eval(File goldFile, File out) {
		int total=0;
		int correct=0;
		SentenceReader<? extends ISentence> gold=ReaderWriterFactory.getReader(goldFile, TagOptions.format);
		SentenceReader<? extends ISentence> pred=ReaderWriterFactory.getReader(out, TagOptions.format);
		
		Iterator<? extends ISentence> gIt=gold.iterator();
		Iterator<? extends ISentence> pIt=pred.iterator();
		
		while(gIt.hasNext()){
			ISentence g=gIt.next();
			ISentence p=pIt.next();
			if(!sameWords(g,p))
				throw new Error("mismatch");
			String[] gPos=g.getTaggerTrainPOS();
			String[] pPos=p.getTaggerPredPOS();
			for(int i=1;i<gPos.length;++i){
				++total;
				if(gPos[i].equals(pPos[i]))
					++correct;
			}
		}
		Accuracy acc=new Accuracy(total,correct);
		System.out.println(acc.toString());
	}
	
	private static boolean sameWords(ISentence g, ISentence p) {
		String[] gForms=g.getForms();
		String[] pForms=p.getForms();
		return Arrays.equals(gForms, pForms);
	}

	public static void testModel(File modelFile,File test,File out) throws ClassNotFoundException, IOException{
		TagModel model=TagModel.loadModel(modelFile);
		IDecoder decoder=model.getDecoder();
		
		testDecoder(decoder,test,out,model.instanceCreator,model.symTab);
	}

	public static void testDecoder(IDecoder decoder, File test, File out,TagInstanceCreator instanceCreator,TagSymbolTable symTab) throws IOException {
		SentenceReader<?> reader=ReaderWriterFactory.getReader(test, TagOptions.format);
		SentenceWriter writer=ReaderWriterFactory.getWriter(out);
		
		for(ISentence s:reader){
			TagInstance inst=instanceCreator.createTestInstance(s, symTab);
			int[] tags=decoder.decodeInstance(inst, symTab);
			String[] pos=new String[tags.length];
			for(int i=1;i<s.getLength();++i){
				pos[i]=symTab.lookupTagString(tags[i]);
			}
			s.setPredPOS(pos);
			writer.write(s);
		}
		
		writer.close();
	}
	
}
