package ims.trans;

import java.util.List;

import ims.data.ISentence;
import ims.data.Instance;
import ims.data.Parse;
import ims.hotcoref.symbols.SymbolTable;
import ims.perceptron.Long2IntInterface;
import ims.trans.transitions.ITransitionFilter;
import ims.util.ArrayFunctions;
import ims.util.Pair;

public abstract class AbstractDecoder {

	public final float[] parameters;
	public final Long2IntInterface l2i;
	public final SymbolTable symTab;
	public final boolean mergeStates;
	public final boolean labeled;
	
	public final InstanceMapping instanceMapping;
	
	public final FeatureExtractor featureExtractor;
	public final ITransitionFilter transitionFilter;
	
	public final String[] deprelLookup;

	public AbstractDecoder(Model m){
		this(m.parameters,m.l2i,m.symTab,m.transitionFilter,m.fe,m.mergeStates,m.labeled,m.instanceMapping);
	}
	
	public AbstractDecoder(float[] parameters, Long2IntInterface l2i, SymbolTable symTab,ITransitionFilter transitionFilter,FeatureExtractor featureExtractor,boolean mergeStates,boolean labeled,InstanceMapping instanceMapping) {
		this.parameters = parameters;
		this.l2i = l2i;
		this.symTab = symTab;
		this.featureExtractor=featureExtractor;
		this.transitionFilter=transitionFilter;
		this.mergeStates=mergeStates;
		this.labeled=labeled;
		this.deprelLookup=symTab.deprels.getInverted();
		this.instanceMapping=instanceMapping;
	}

	protected double[] scoreTransitions(State s,Instance inst,int[] trans) {
		double[] r=new double[trans.length];
		long[] sink=featureExtractor.sink;
		int f=featureExtractor.extractFeatures(s, sink, inst);
		ArrayFunctions.shiftAllLeftByBits(sink, featureExtractor.bA, f);
		for(int i=0;i<trans.length;++i){
			double transScore=getTransitionScore(trans[i],sink,f);
			r[i]=s.totalScore+transScore;
		}
		return r;
	}

	public double getTransitionScore(int transID,long[] features,int f){
		double score=0.d;
		for(int i=0;i<f;++i){
			long l=features[i];
			l|=transID;
			score+=parameters[l2i.l2i(l)];
		}
		return score;
	}
	
	public Pair<int[],String[]> state2tree(State s){
		Pair<int[],String[]> tree=s.getPredTree(deprelLookup);
		return tree;
	}
	
	public abstract State parse(ISentence s);

	public abstract Pair<State, List<Parse>> parseNBest(ISentence s);
	
}
