package ims.trans;

import java.io.Serializable;

import ims.data.Instance;
import ims.hotcoref.symbols.AbstractSymbolMapping;
import ims.hotcoref.symbols.SymbolTable;
import ims.perceptron.Long2IntInterface;
import ims.perceptron.ParametersFloat;

public abstract class FeatureExtractor implements Serializable{
	private static final long serialVersionUID = 3521203190028337055L;

	public static final int NONE=AbstractSymbolMapping.NONE_IDX;
	public static final int[] NONE_ARR=new int[]{AbstractSymbolMapping.NONE_IDX};
	public static final short[] NONE_SHORT_ARR=new short[]{AbstractSymbolMapping.NONE_IDX};
	
	public final long[] sink=new long[Options.featureSinkSize];
	
	protected final int
		bF, //Bits for form and lemma
		bT, //Bits for pos tag
		bM, //Bits for morphology
		bL, //Bits for deprels
		bS, //Bits for supertags
		bSH; //Bits for shapes
	protected final int bA; //Bits for actions
	
	protected final int 
		bD; //bits for dist
	
	protected final int
		bCS; //bits for char shapes
	
	public FeatureExtractor(SymbolTable symTab){
		this(symTab.forms.getBits(),
				symTab.tags.getBits(),
				symTab.feats.getBits(),
				symTab.transitions.getBits(),
				symTab.deprels.getBits(),
				symTab.stags==null ?0:symTab.stags.getBits(),
				symTab.shapes==null?0:symTab.shapes.getBits()
				);
	}
	
	public FeatureExtractor(int bF,int bT,int bM,int bA,int bL,int bS,int bSH){
		this.bF=bF;
		this.bT=bT;
		this.bM=bM;
		this.bA=bA;
		this.bL=bL;
		this.bS=bS;
		this.bSH=bSH;
		this.bD=BITS_BUCKETS;
		this.bCS=BITS_CHAR_SHAPE;
	}
	
	public static final int BITS_CHAR_SHAPE=3; //Max value currently 6 --> fits in 3 bits
	
	public static final int BITS_BUCKETS=5;
	public static final int getIntBucket(int i){
		if(i<0)
			return 4;
		if(i==0)
			return 5;
		if(i==1)
			return 6;
		if(i==2)
			return 7;
		if(i==3)
			return 8;
		if(i<6)
			return 9;
		if(i<10)
			return 10;
		if(i<20)
			return 11;
		return 12;
	}
	
	public static final int getSentLengthBucket(int i){
		return getValencyBucket(i);
	}
	
	public static final int getValencyBucket(int i){
		if(i<0)
			return 4;
		if(i==0)
			return 5;
		if(i==1)
			return 6;
		if(i==2)
			return 7;
		if(i==3)
			return 8;
		if(i<6)
			return 9;
		if(i<10)
			return 10;
		if(i<20)
			return 11;
		if(i<40)
			return 12;
		if(i<60)
			return 13;
		return 14;
	}
	
	public static final int getStackSizeBucket(int i){
		if(i<0)
			return 4;
		if(i<=10)
			return 4+i; //since i is never 0, this works out
		return 15;
	}
	
	public abstract int extractFeatures(State s,long[] sink,Instance inst);

	public void initWeights(SymbolTable symTab,ParametersFloat parameters,Long2IntInterface l2i) {
		//Do nothings
	}
	
}
