package ims.trans;

import java.util.List;

import ims.data.ISentence;
import ims.data.Instance;
import ims.data.Parse;
import ims.hotcoref.symbols.SymbolTable;
import ims.perceptron.Long2IntInterface;
import ims.trans.transitions.ITransition;
import ims.trans.transitions.ITransitionFilter;
import ims.util.Pair;
import ims.util.ProbSampler;

public class SamplingDecoder extends AbstractDecoder {

	private final ProbSampler probSampler;
	private final SamplingStoppingCriterion stopper;
	
	public int totalSamples=0;
	
	public SamplingDecoder(Model m){
		this(m.parameters,m.l2i,m.symTab,m.transitionFilter,m.fe,m.mergeStates,m.labeled,m.instanceMapping,SamplingStoppingCriterion.get(m.stoppingCriterion, m.sampleK));
	}
	
	public SamplingDecoder(float[] parameters, Long2IntInterface l2i, SymbolTable symTab,ITransitionFilter transitionFilter,FeatureExtractor featureExtractor,boolean mergeStates,boolean labeled,InstanceMapping instanceMapping,SamplingStoppingCriterion stopper) {
		super(parameters, l2i, symTab, transitionFilter, featureExtractor, labeled, labeled, instanceMapping);
		this.probSampler=new ProbSampler(Options.sampleSeed);
		this.stopper=stopper;
	}
	
	public State findBestSample(Instance inst){
		State best=null;
		int totalRestarts=0;
		int totalSinceLastIncrease=0;
		while(!stopper.stop(totalRestarts, totalSinceLastIncrease)){
			++totalRestarts;
			++totalSinceLastIncrease;
			State s=sampleState(inst);
			if(best==null || best.totalScore<s.totalScore){
				best=s;
				totalSinceLastIncrease=0;
			}
		}
		totalSamples+=totalRestarts;
		return best;
	}
	
	public State sampleState(Instance inst){
		State s=State.getInitialState(inst.form.length, labeled, null);
		while(!s.isTerminal()){
			int[] trans=transitionFilter.getAllowedTransitions(s,inst);
			double[] newScores=scoreTransitions(s,inst,trans);
			int t=probSampler.sampleEntry(newScores);
			ITransition tr=symTab.transition2label[trans[t]];
			s=tr.apply(s, newScores[t]);
		}
		return s;
	}

	@Override
	public State parse(ISentence s) {
		Instance inst=instanceMapping.createTestInstance(s, symTab);
		State state=findBestSample(inst);
		return state;
	}

	@Override
	public Pair<State, List<Parse>> parseNBest(ISentence s) {
		throw new Error("nbest parsing not implemented for sampling decoder");
	}
	
}
