package ims.trans;

public abstract class SamplingStoppingCriterion {

	public static enum Criteria { FixedK, AdaptiveK }
	
	public abstract boolean stop(int totalIter,int totalSinceLastIncrease);
	
	public static SamplingStoppingCriterion get(Criteria c,int k){
		switch (c){
		case AdaptiveK: return new AdaptiveKStopping(k);
		case FixedK:    return new FixedKStopping(k);
		default:		throw new Error("not implmented");
		}
	}
	
	public static class AdaptiveKStopping extends SamplingStoppingCriterion {
		private final int k;
		
		public AdaptiveKStopping(int k){
			this.k=k;
		}
		
		@Override
		public boolean stop(int totalIter, int totalSinceLastIncrease) {
			if(totalSinceLastIncrease>k)
				return true;
			else
				return false;
		}
	}
	
	public static class FixedKStopping extends SamplingStoppingCriterion {
		private final int k;
		
		public FixedKStopping(int k){
			this.k=k;
		}
		
		@Override
		public boolean stop(int totalIter, int totalSinceLastIncrease) {
			if(totalIter>k)
				return true;
			else
				return false;
		}
	}
	
}
