package ims.trans;

public class SimpleBuffer implements IBuffer {

	private final int nxt;
	private final int last;

	public SimpleBuffer(int next,int last) {
		this.nxt=next;
		this.last=last;
	}

	@Override
	public IBuffer pushBack(State s) {
		return new PushbackBuffer(nxt,last,s);
	}

	@Override
	public int size() {
		return last-nxt+1;
	}

	@Override
	public int get(int n) {
		if(size()-n<=0)
			return -1;
		else
			return nxt+n;
	}

	@Override
	public IBuffer advance() {
		return getBuffer(nxt+1,last);
	}

	@Override
	public State peekPushedBack() {
		return null;
	}
	

	@Override
	public boolean hasSwappedItems() {
		return false;
	}
	
	/*
	 * An empty buffer
	 */
	
	public static final IBuffer EMPTY_BUFFER=new IBuffer(){
		@Override
		public IBuffer pushBack(State s) {
			return new PushbackBuffer(1,0,s);
		}
		@Override
		public int size() {
			return 0;
		}
		@Override
		public int get(int n) {
			return -1;
		}
		@Override
		public IBuffer advance() {
			throw new Error("Don't think this should happen?");
		}

		@Override
		public State peekPushedBack() {
			throw new Error("Don't think this should happen?");
		}
		@Override
		public boolean hasSwappedItems() {
			return false;
		}
		@Override
		public boolean equals(IBuffer b) {
			return b.size()==0;
		}
		@Override
		public boolean equals(SimpleBuffer other) {
			return other.size()==0;
		}
		@Override
		public boolean equals(PushbackBuffer other) {
			return other.size()==0;
		}
		@Override
		public IBuffer deepCopy() {
			return this;
		}
		@Override
		public int getNoPushedBack() {
			return 0;
		}
		
		public String toString() {
			return "Empty Buffer";
		}
	};
	
	
	
	/*
	 * Cache stuff
	 */
	
	private static final int CACHE_DEFAULT_SIZE=600;
	private static IBuffer[][] cache=new IBuffer[CACHE_DEFAULT_SIZE+1][CACHE_DEFAULT_SIZE];
	static {
		for(int nxt=1;nxt<CACHE_DEFAULT_SIZE+1;++nxt){
			for(int last=1;last<nxt;++last)
				cache[nxt][last]=EMPTY_BUFFER;
			for(int last=nxt;last<CACHE_DEFAULT_SIZE;++last)
				cache[nxt][last]=new SimpleBuffer(nxt,last);
		}
		cache[1][0]=EMPTY_BUFFER;
	}
	
	public static IBuffer getBuffer(int nxt,int last){
		if(last<CACHE_DEFAULT_SIZE)
			return cache[nxt][last];
		else
			return new SimpleBuffer(nxt,last);
	}

	@Override
	public boolean equals(IBuffer b) {
		return b.equals(this);
	}
	
	public boolean equals(SimpleBuffer other){
		return nxt==other.nxt && last==other.last;
	}
	
	public boolean equals(PushbackBuffer other){
		return other.equals(this);
	}

	@Override
	public IBuffer deepCopy() {
		//Here we cheat, cause they're all the same anyway
		return this;
	}

	@Override
	public int getNoPushedBack() {
		return 0;
	}

	public String toString() {
		return "["+nxt+".."+last+"]";
	}
	
	public static void main(String[] args){
		for(int a=1;a<501;++a){
			int min=999;
			int max=-1;
			for(int b=1;b<500;++b){
				if(cache[a][b]!=null){
					if(b<min)
						min=b;
					if(b>max)
						max=b;
				}
			}
			System.out.printf("a: %4d, min: %4d, max: %4d\n",a,min,max);
		}
	}
}
