package ims.trans;

import java.io.File;
import java.io.IOException;
import java.util.List;

import ims.data.CoNLL09Sentence;
import ims.data.ISentence;
import ims.data.NBestCoNLL09Sentence;
import ims.data.Parse;
import ims.io.ReaderWriterFactory;
import ims.io.SentenceReader;
import ims.io.SentenceWriter;
import ims.trans.util.Eval;
import ims.util.Accuracy;
import ims.util.DBO;
import ims.util.Pair;
import ims.util.ZipUtil;

public class Test implements AbstractTrainer.Tester{

	public static void main(String[] args) throws IOException, ClassNotFoundException{
		Options options=new Options(args);
		DBO.println("Loading model");
		Model m=ZipUtil.loadObjectFromEntry(Model.ZIP_FILE_ENTRY, options.model, Model.class);
		DBO.println("Done.");
		AbstractDecoder d;
		if(options.sampleDecode)
			d=new SamplingDecoder(m);
		else {
			d=new Decoder(m);
			if(options.overRideBeam>0)
				((Decoder) d).overrideBeam(options.overRideBeam);			
		}
		SentenceReader<?> reader=ReaderWriterFactory.getReader(options.test, Options.format.sentenceClass,m.reversed);
		SentenceWriter writer=ReaderWriterFactory.getWriter(options.out,m.reversed);
		DBO.println("Parsing");
		test(d, reader, writer, options.outputNBest);
		writer.close();
		DBO.println("Done");
		if(options.gold!=null){
			final Pair<Accuracy,Accuracy> pair;
			if(options.outputNBest){
				SentenceReader<CoNLL09Sentence> goldReader=ReaderWriterFactory.getReader(options.gold, CoNLL09Sentence.class);
				SentenceReader<NBestCoNLL09Sentence> predReader=ReaderWriterFactory.getReader(options.out,NBestCoNLL09Sentence.class);
				pair=Eval.getUASandLAS(goldReader, predReader);
			} else {
				pair=Eval.getUASandLAS(options.out, options.gold, Options.format);
			}
			System.out.println("UAS: "+pair.getLeft().toString() + "   LAS: "+pair.getRight().toString());
		}
		options.done();
	}

	public static void test(AbstractDecoder d, SentenceReader<?> reader, SentenceWriter writer)throws IOException {
		test(d,reader,writer,false);
	}
	
	public static void test(AbstractDecoder d, SentenceReader<?> reader, SentenceWriter writer,boolean outputNBest)throws IOException {
		int sentCount=0;
		int erase=0;
		DBO.printNoPrefix("Sent: ");
		long start=System.currentTimeMillis();
		for(ISentence s:reader){
			final State bestState;
			final ISentence outS;
			if(outputNBest){
				Pair<State,List<Parse>> parsed=d.parseNBest(s);
				bestState=parsed.getLeft();
				outS=s.createNBestSentence(parsed.getRight());
			} else {
				bestState=d.parse(s);
				outS=s;
			}
			Pair<int[],String[]> p=d.state2tree(bestState);
			int[] heads=p.getLeft();
			String[] lbl=p.getRight();
			outS.setPredHeads(heads);
			outS.setPredLabels(lbl);
			writer.write(outS);
			sentCount++;
			if(sentCount%111==0 || Options.DEBUG)
				erase=DBO.eraseAndPrint(erase, getInfoString(sentCount,System.currentTimeMillis()-start));
		}
		erase=DBO.eraseAndPrint(erase, getInfoString(sentCount,System.currentTimeMillis()-start));
		DBO.println();
	}

	private static String getInfoString(int sentCount, long l) {
		long tpi=l/sentCount;
		return String.format("%-5d   (%d/s)", sentCount,tpi);
	}

//	public static void applyDecoder(AbstractDecoder d, SentenceReader<?> reader, SentenceWriter writer) throws IOException {
//		test(d,reader,writer);
//	}

	@Override
	public void applyDecoder(AbstractDecoder decoder, SentenceReader<?> reader,	File output, boolean reversed) throws IOException {
		SentenceWriter writer=ReaderWriterFactory.getWriter(output, reversed);
		test(decoder,reader,writer);
	}

	@Override
	public void evalTest(File goldFile, File predFile) {
		Pair<Accuracy,Accuracy> pair=Eval.getUASandLAS(predFile, goldFile, Options.format);
		System.out.println("UAS: "+pair.getLeft().toString() + "   LAS: "+pair.getRight().toString());
	}
	
}
