package ims.trans.transitions;

import ims.data.Instance;
import ims.hotcoref.symbols.SymbolTable;
import ims.oracle.Configuration.TR;
import ims.trans.State;
import ims.util.IntPair;
import ims.util.MutableInt;

import java.util.List;
import java.util.Map;
import java.util.Set;
import java.util.TreeSet;

public abstract class AbstractPOSTransitionFilter implements ITransitionFilter {
	private static final long serialVersionUID = 770874534368108703L;

	protected int extractTR(TR tr,SymbolTable symTab){
		int t=-1;
		for(int i=2;i<symTab.transition2label.length;++i){
			ITransition itr=symTab.transition2label[i];
			if(itr.getTR()==tr){
				t=itr.getId();
				break;
			}
		}
		if(t==-1)
			throw new Error("Found no "+tr.toString()+"??? This should not happen.");
		return t;
	}
	
	protected void register(Map<IntPair,Set<Integer>> seenXA,int s0Pos,int s1Pos,int t) {
		IntPair ip=IntPair.intPairPool.get(s0Pos, s1Pos);
		Set<Integer> s=seenXA.get(ip);
		if(s==null){
			s=new TreeSet<Integer>();
			seenXA.put(ip, s);
		}
		s.add(t);
	}
	
	protected int countArcs(SymbolTable symTab,List<Instance> instances,Map<Integer, MutableInt> LAcounts,Map<Integer, MutableInt> RAcounts,Map<IntPair, Set<Integer>> seenLA,Map<IntPair, Set<Integer>> seenRA) {
		int maxPos=0;
		ITransition[] itr=symTab.transition2label;
		
		for(Instance is:instances){
			State state=State.getInitialState(is.form.length, true, is.head);
			for(int t:is.transSequence){
				ITransition i=itr[t];
				if(i.isArcTransition()){
					
					int s0=state.peekStack(0);
					int s1=state.peekStack(1);
					
					int s0Pos=is.tag[s0];
					int s1Pos=is.tag[s1];
//					int lbl=is.lbl[dep];
					
					maxPos=Math.max(maxPos, Math.max(s0Pos,s1Pos));
					
					boolean isLA=i.getBasicTransitionLabel().startsWith("LA");
					if(isLA){
						MutableInt.incrementMapValue(LAcounts, t);
						register(seenLA,s0Pos,s1Pos,t);
					} else {
						MutableInt.incrementMapValue(RAcounts, t);
						register(seenRA,s0Pos,s1Pos,t);						
					}
				}
				state=i.apply(state, 0);
			}
		}
		return maxPos;
	}
	
	@Override
	public boolean isPOSFiltered(){
		return false;
	}
	
}
