package ims.trans.transitions;

import java.util.Collections;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Set;
import java.util.TreeMap;
import java.util.TreeSet;

import ims.data.Instance;
import ims.hotcoref.symbols.SymbolTable;
import ims.oracle.Configuration.TR;
import ims.trans.State;
import ims.util.IntPair;
import ims.util.MutableInt;

public class POSArcStandardTransitionFilter extends AbstractPOSTransitionFilter {
	private static final long serialVersionUID = -1494813112632745336L;

	//Shift
	final int[] SH; //1
	
	//The complicated ones
	final int[][][] SH_RA_LA;
	final int[][][] SH_RA;
	final int[][][] RA;
	final int[][][] RA_LA;
	
	//With default labels
	final int[] d_SH_RA_LA; //3
	final int[] d_SH_RA;    //2
	final int[] d_RA;       //1
	final int[] d_RA_LA;    //2
	
	public POSArcStandardTransitionFilter(SymbolTable symTab,List<Instance> instances){
		//Deal with SHIFT
		int sh=extractTR(TR.SH,symTab);
		this.SH=new int[]{sh};
		
		//Now loop over instances a do counts and records
		Map<Integer,MutableInt> LAcounts=new TreeMap<Integer,MutableInt>();
		Map<Integer,MutableInt> RAcounts=new TreeMap<Integer,MutableInt>();
		
		Map<IntPair,Set<Integer>> seenLA=new HashMap<IntPair,Set<Integer>>();
		Map<IntPair,Set<Integer>> seenRA=new HashMap<IntPair,Set<Integer>>();
		
		//Populate the maps
		int maxPos = countArcs(symTab, instances, LAcounts, RAcounts, seenLA, seenRA);

		final int defaultLA=MutableInt.getMostFrequent(LAcounts);
		final int defaultRA=MutableInt.getMostFrequent(RAcounts);
		
		//Assign defaults
		this.d_SH_RA_LA=new int[]{sh,defaultRA,defaultLA};
		this.d_SH_RA   =new int[]{sh,defaultRA};
		this.d_RA      =new int[]{defaultRA};
		this.d_RA_LA   =new int[]{defaultRA,defaultLA};
		
		
		//Now deal with the arc transitions
		
		//Collect all head/dep pairs
		Set<IntPair> hdPairs=new TreeSet<IntPair>();
		hdPairs.addAll(seenLA.keySet());
		hdPairs.addAll(seenRA.keySet());
		
		//Now create these arrays (finally...)
		SH_RA_LA = new int[maxPos+1][maxPos+1][];
		SH_RA    = new int[maxPos+1][maxPos+1][];
		RA       = new int[maxPos+1][maxPos+1][];
		RA_LA    = new int[maxPos+1][maxPos+1][];

		for(IntPair hdp:hdPairs){
			int s0Pos=hdp.i1;
			int s1Pos=hdp.i2;

			Set<Integer> la=seenLA.get(hdp);
			Set<Integer> ra=seenRA.get(hdp);
			if(la==null)
				la=Collections.singleton(defaultLA);
			if(ra==null)
				ra=Collections.singleton(defaultRA);

			Set<Integer> union=new TreeSet<Integer>();
			union.addAll(la);
			union.addAll(ra);

//			int l=la.size();
			int r=ra.size();
			int u=union.size();

			SH_RA_LA[s0Pos][s1Pos]=new int[u+1]; int a=0;
			RA_LA[s0Pos][s1Pos]   =new int[u];   int b=0;
			SH_RA[s0Pos][s1Pos]   =new int[r+1]; int c=0;
			RA[s0Pos][s1Pos]      =new int[r];   int d=0;
			
			SH_RA_LA[s0Pos][s1Pos][a++]=sh;
			SH_RA[s0Pos][s1Pos][c++]=sh;
			
			for(Integer i:union){
				SH_RA_LA[s0Pos][s1Pos][a++]=i.intValue();
				RA_LA[s0Pos][s1Pos][b++]=i.intValue();
			}
			for(Integer i:ra){
				SH_RA[s0Pos][s1Pos][c++]=i.intValue();
				RA[s0Pos][s1Pos][d++]=i.intValue();
			}
			if(SH_RA_LA[s0Pos][s1Pos].length!=a)
				throw new Error("here");
			if(RA_LA[s0Pos][s1Pos].length!=b)
				throw new Error("here2");
			if(SH_RA[s0Pos][s1Pos].length!=c)
				throw new Error("here3");
			if(RA[s0Pos][s1Pos].length!=d)
				throw new Error("here4");
		}
	}
	


	@Override
	public int[] getAllowedTransitions(State s, Instance inst) {
		boolean canArc=s.hasMinTwoStackElements();
		if(!canArc)
			return SH;
		boolean canLA=s.peekStack(1)!=0; //can't LA if root is s1
		boolean canShift=s.bufferSize()>0;
		int s0=s.peekStack(0);
		int s1=s.peekStack(1);
		int s0Pos=inst.tag[s0];
		int s1Pos=inst.tag[s1];
		final int[] r;
		if(canLA){ //Both RA and LA
			if(canShift)
				r=SH_RA_LA[s0Pos][s1Pos];
			else
				r=RA_LA[s0Pos][s1Pos];
		} else {   //Only RA
			if(canShift)
				r=SH_RA[s0Pos][s1Pos];
			else
				r=RA[s0Pos][s1Pos];
		}
		//Return if we have seen this pos pair
		if(r!=null)
			return r;
		
		//Otherwise we have to return default
		if(canLA){
			if(canShift)
				return d_SH_RA_LA;
			else
				return d_RA_LA;
		} else {
			if(canShift)
				return d_SH_RA;
			else
				return d_RA;
		}
	}

}
