package ims.trans.util;

import java.util.ArrayList;
import java.util.List;

import ims.data.ISentence;
import ims.io.ReaderWriterFactory;
import ims.io.SentenceReader;
import ims.trans.Options;
import ims.util.Util;

public class CheckIsTree {

	public static void main(String[] args){
		Options options=new Options(args);
		SentenceReader<?> trainReader=ReaderWriterFactory.getTrainReader(options);
		
		int sID=0;
		List<Integer> missingHeads=new ArrayList<>();
		List<Integer> cycles=new ArrayList<>();
		for(ISentence s:trainReader){
			++sID;
//			int[] heads=s.getPredHeads();
			int[] heads=s.getGoldHeads();
			if(hasMissingHeads(heads)){
				missingHeads.add(sID);
			}
			if(hasCycles(heads)){
				cycles.add(sID);
			}
		}
		
		if(missingHeads.size()>0){
			System.out.println("Missing heads in the following sentences:");
			System.out.println(Util.joinStrings(missingHeads, ", "));
		}
		
		if(cycles.size()>0){
			System.out.println("Found cycles in the following sentences:");
			System.out.println(Util.joinStrings(cycles, ", "));
		}
	}

	private static boolean hasCycles(int[] heads) {
		boolean[] visited=new boolean[heads.length];
		for(int dep=1;dep<heads.length;++dep){
			if(visited[dep])
				continue;
			for(int d=dep,i=0;;++i){
				int h=heads[d];
				if(h==0) //reached root, all good
					break;
				if(h<0) //missing heads, break here
					break;
				if(i>heads.length) //If we have walked around in the graph more than heads.length steps without encountering root, then we certainly have a cycle
					return true;
				visited[d]=true;
				d=h;
			}
		}
		return false;
	}

	private static boolean hasMissingHeads(int[] heads) {
		for(int i=1;i<heads.length;++i)
			if(heads[i]<0)
				return true;
		return false;
	}
		
	
}
