package ims.trans.util;

import java.io.IOException;

import ims.data.ISentence;
import ims.io.ReaderWriterFactory;
import ims.io.SentenceWriter;
import ims.trans.Decoder;
import ims.trans.Model;
import ims.trans.Options;
import ims.trans.State;
import ims.trans.transitions.ITransition;
import ims.util.DBO;
import ims.util.Pair;
import ims.util.ZipUtil;

public class ParsePrintSeqs {

	public static void main(String[] args) throws ClassNotFoundException, IOException{
		Options options=new Options(args);
		DBO.setOut(System.err);
		DBO.println("Loading model");
		Model m=ZipUtil.loadObjectFromEntry(Model.ZIP_FILE_ENTRY, options.model, Model.class);
		DBO.println("Done.");
		Decoder d=new Decoder(m);
		Iterable<? extends ISentence> reader=ReaderWriterFactory.getReader(options.test, options.format);
		SentenceWriter writer=null;
		if(options.out!=null)
			writer=ReaderWriterFactory.getWriter(options.out);

		for(ISentence s:reader){
			State state=d.parse(s);
			ITransition[] trs=State.getTransitionSequence(state);
			StringBuilder sb=new StringBuilder();
			sb.append(trs[0].getFullTransitionLabel());
			for(int i=1;i<trs.length;++i)
				sb.append(",").append(trs[i].getFullTransitionLabel());
			System.out.println(sb.toString());
			if(writer!=null){
				Pair<int[],String[]> p=d.state2tree(state);
				int[] heads=p.getLeft();
				String[] lbl=p.getRight();
				s.setPredHeads(heads);
				s.setPredLabels(lbl);
				writer.write(s);
			}
		}
		if(writer!=null)
			writer.close();
	}

}
