package ims.trans.util;

import ims.data.ISentence;
import ims.io.ReaderWriterFactory;
import ims.io.SentenceReader;
import ims.oracle.IOracle;
import ims.oracle.OracleFactory;
import ims.trans.Options;
import ims.util.NonProjInfo;
import ims.util.Util;

import java.util.Arrays;
import java.util.List;

public class PrintNPAndSwapInfo {

	private static final String DELIM = "  ========================================\n";

	public static void main(String[] args) throws Exception{
		Options options=new Options(args);
		IOracle oldSwapOracle=OracleFactory.getStaticOracle(Options.TransitionSystem.ArcSOldSwap);
		IOracle newSwapOracle=OracleFactory.getStaticOracle(Options.TransitionSystem.ArcSNewSwap);
		IOracle optSwapOracle=OracleFactory.getStaticOracle(Options.TransitionSystem.ArcSOptimalSwap);
//		Iterator<ISentence> it=new StreamIteratorReader(System.in,Format.CoNLL06);
		SentenceReader<?> reader=ReaderWriterFactory.getTrainReader(options);
		int count=0;
//		while(it.hasNext()){
//			if(i>=options.count)
//				break;
//			ISentence s=it.next();
		for(ISentence s:reader){
			int[] head=s.getGoldHeads();
			NonProjInfo npi=new NonProjInfo(head);
			ArcStandardPrecedence asp=new ArcStandardPrecedence(head);
			List<String> oldSeq=oldSwapOracle.getCanonicalSequence(head, null, false);
			List<String> newSeq=newSwapOracle.getCanonicalSequence(head, null, false);
			List<String> optSeq=optSwapOracle.getCanonicalSequence(head, null, false);
			
			int swOld=countSwaps(oldSeq);
			int swNew=countSwaps(newSeq);
			int swOpt=countSwaps(optSeq);
			String projOrder=PrintProjectiveOrder.getProjectiveOrderFormatted(s);
			System.out.println(projOrder);
			System.out.printf("I:  %s\n",Arrays.toString(npi.intervalDegree));
			System.out.printf("L:  %s\n",Arrays.toString(npi.levelType));
			System.out.println("For np arcs:");
			for(int i=1;i<head.length;++i){
				if(!npi.isNonProjectiveEdge[i])
					continue;
				System.out.printf("%2d    LS: %-10s   CR: %-10s   G: %-10s\n",i,Arrays.toString(npi.levelSignatures[i]),Arrays.toString(npi.componentRoots[i]),Arrays.toString(npi.gaps[i]));
			}
			System.out.println();
			System.out.printf("SWAPS    old: %d    new: %d    opt: %d\n",swOld,swNew,swOpt);
			System.out.println();
			System.out.println("ASP:");
			for(int i=0;i<asp.p.size();++i){
				System.out.printf("%-3d   %s\n",i,Util.joinStrings(asp.p.get(i), ","));
			}
			System.out.println(DELIM);
			count++;
		}
	}

	private static int countSwaps(List<String> l) {
		int c=0;
		for(String q:l)
			if(q.equals("SW"))
				++c;
		return c;
	}
	
}
