package ims.trans.util;

import java.util.ArrayList;
import java.util.List;

import ims.data.Instance;
import ims.hotcoref.symbols.SymbolTable;
import ims.io.DocumentReader;
import ims.oracle.IOracle;
import ims.oracle.OracleFactory;
import ims.oracle.Configuration.TR;
import ims.trans.InstanceMapping;
import ims.trans.Options;
import ims.trans.Options.TransitionSystem;
import ims.util.Util;

public class SBDocOracleSeqAnalysis {

	public static void main(String[] args){
		Options options=new Options(args);
		
		IOracle oracle=OracleFactory.getStaticOracle(TransitionSystem.ArcSNewSwap);

		SymbolTable symTab=new SymbolTable(options.useStag,true,options.useShape);

		InstanceMapping instanceMapping=new InstanceMapping(options.useStag,options.useStack,options.useShape);

		DocumentReader<?> reader=DocumentReader.getReader(Options.format.sentenceClass, options.train);

//		List<Instance> instances = createInstances(options, reader, oracle, symTab,	instanceMapping);
//		int instCount=
		symTab.registerSimple(reader.getSentenceIterator(), options.count, options.labeled);
		symTab.addSwapTransition();
		symTab.addSBTransition();
		List<Instance> early = createInstances(reader,oracle,symTab,instanceMapping,true,options.labeled);
		List<Instance> late  = createInstances(reader,oracle,symTab,instanceMapping,false,options.labeled);
		symTab.freezeMappings();
		
		
		int[] distBuckets=new int[100];
		
		for(int i=0;i<early.size();++i){
			Instance e=early.get(i);
			Instance l=late.get(i);
			
			countDists(e,l,distBuckets,symTab.transitions.lookup(TR.SB.toString()));
			
//			List<String> eSeq=getSeq(e,symTab);
//			List<String> lSeq=getSeq(l,symTab);
//			printSeq(eSeq);
//			printSeq(lSeq);
//			System.out.println();
		}
		
		for(int i=0;i<distBuckets.length;++i){
			if(distBuckets[i]>0){
				System.out.printf("%-3d\t%,d\n",i,distBuckets[i]);
			}
		}
	}

	private static void countDists(Instance ei, Instance li, int[] distBuckets,int sbTR) {
		int[] eSeq=ei.transSequence;
		int[] lSeq=li.transSequence;
		if(eSeq.length!=lSeq.length)
			throw new Error("!");
		
		int lastE=-1;
		
		for(int i=0;i<eSeq.length;++i){
			int e=eSeq[i];
			int l=lSeq[i];
			if(e==sbTR)
				lastE=i;
			if(l==sbTR){
				int dist=i-lastE;
				distBuckets[dist]++;
			}
		}
	}

	private static void printSeq(List<String> lSeq) {
		System.out.println(Util.joinStrings(lSeq, ", "));
	}

	private static List<String> getSeq(Instance e, SymbolTable symTab) {
		List<String> r=new ArrayList<>();
		for(int i:e.transSequence)
			r.add(symTab.transition2label[i].getFullTransitionLabel());
		return r;
	}

	private static List<Instance> createInstances(DocumentReader<?> reader,	IOracle oracle, SymbolTable symTab,	InstanceMapping instanceMapping, boolean early,boolean labeled) {
		List<Instance> l=instanceMapping.createTrainInstances(reader, symTab, oracle, labeled, early);
		return l;
	}
}
