package ims.util;

import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.File;
import java.io.IOException;
import java.util.ArrayList;
import java.util.List;

public class CVSplit {

	public static void main(String[] args) throws IOException{
		int folds=Integer.parseInt(args[0]);
		
		List<List<String>> sentences=readSentences();
		List<List<List<String>>> splits=cvSplit(sentences,folds);
		
		for(int i=0;i<folds;++i){
			BufferedWriter train=IOUtils.getBufferedWriter(new File("train-"+i));
			BufferedWriter test=IOUtils.getBufferedWriter(new File("test-"+i));
			
			for(int q=0;q<folds;++q){
				List<List<String>> split=splits.get(q);
				if(q==i){ //goes in test
					writeSentences(split,test);
				} else {  //goes in train
					writeSentences(split,train);
				}
			}
			train.close();
			test.close();
		}
	}

	private static void writeSentences(List<List<String>> split, BufferedWriter out) throws IOException {
		for(List<String> l:split){
			for(String s:l){
				out.write(s);
				out.write("\n");
			}
			out.write("\n");
		}
	}

	private static List<List<String>> readSentences() throws IOException {
		List<List<String>> sentences=new ArrayList<List<String>>();
		BufferedReader reader=IOUtils.getSTDINBufferedReader();
		List<String> next=new ArrayList<String>();
		String line;
		while((line=reader.readLine())!=null){
			if(line.isEmpty()){
				sentences.add(next);
				next=new ArrayList<String>();
			} else {
				next.add(line);
			}
		}
		if(!next.isEmpty())
			sentences.add(next);
		return sentences;
	}

	public static <T> List<List<T>> cvSplit(List<T> items,int folds){
		List<List<T>> r=new ArrayList<List<T>>(); 
		if(items.size()<folds)
			throw new Error("can't cv with fewer instances than folds");
		int br=items.size()/folds;
		int b=0;
		int n=br;
		for(int i=0;i<folds-1;++i){
			r.add(items.subList(b, n));
			b=n;
			n+=br;
		}
		r.add(items.subList(b, items.size()));
		return r;
	}
	
}
