package ims.util;

import java.util.Arrays;

public class IntPacker {
	public static final int BITS_PER_ITEM=16;
	public static final int ITEMS_PER_LONG=64/BITS_PER_ITEM;
	
	private final long[] l;
	
	public IntPacker(int length){		
		int size=length/ITEMS_PER_LONG;
		if(length%ITEMS_PER_LONG>0)
			size+=1;
		l=new long[size];
	}

	private IntPacker(IntPacker o){
		this.l=Arrays.copyOf(o.l, o.l.length);
	}
	
	public boolean equals(Object other){
		return equals((IntPacker) other);
	}
	
	public boolean equals(IntPacker other){
		return Arrays.equals(l, other.l);
	}
	
	public void set(int idx,int value){
		int a=idx/ITEMS_PER_LONG; //Index in the array l
		int b=idx%ITEMS_PER_LONG; //Position in long
		long n=value+1; //This way we can handle setting the value 0
		n<<=(b*BITS_PER_ITEM);
		l[a]|=n; 
	}
	
	public int get(int idx){
		int a=idx/ITEMS_PER_LONG;
		int b=idx%ITEMS_PER_LONG;
		long n=l[a]>>(b*BITS_PER_ITEM);
		n&=0xffff; //Take last sixteen bits
		return (int) n-1;
	}

	public IntPacker deepCopy() {
		return new IntPacker(this);
	}
	
	public StringBuilder appendSignature(StringBuilder sb){
		for(int i=0;i<l.length;++i)
			sb.append(Long.toHexString(l[i]));
		return sb;
	}
	
	public int getTotalMaxLength(){
		return ITEMS_PER_LONG*l.length;
	}
	
	
	public static void main(String[] args){
		IntPacker ip=new IntPacker(10);
		ip.set(1, 2);    System.out.println(ip.get(1));
		                 System.out.println(ip.get(3)); //Not set, should be -1
		ip.set(4, 5);    System.out.println(ip.get(4));
		ip.set(9, 259);  System.out.println(ip.get(9));
		ip.set(0, 0);    System.out.println(ip.get(0));
		//Test overflow
		System.out.println();
		IntPacker ip2=new IntPacker(10);
		ip2.set(1, 133333); //Which is above 2^16
		System.out.println(ip2.get(0));
		System.out.println(ip2.get(1));
		System.out.println(ip2.get(2)); //Overflows into this bucket, not good, but works as long as nobody breaks the implicit contract
	}
}
